'２次元電磁界解析V84の標準ﾓｼﾞｭｰﾙ      ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module ElMa2DCom
    '２次元および軸対称体の電磁界解析
    'アイソメトリック四角形要素使用
    '自動メッシュ生成は「ＦＥＭ断面データ」ソフトによる
    '要素の節点番号の入力は、反時計回り

    'V71での改良点・・・後処理の解析結果の主要値表示でクリップボードにコピーを追加
    'V72での改良点・・・各画面でボタンを読みやすい色に改善
    'V73での改良点・・・Visual Basic 2017に変更
    'V74での改良点・・・構造図画面で「Ctrl+マウス」の矩形ドラッグによる一括設定画面で
    '                   対象の一括削除を追加
    'V75での改良点・・・解析計算開始前に入力データのエラーチェックを追加
    'V76での改良点・・・プログラムファイルの文字コードをUTF-8に統一、
    '                   データファイルの文字コードを標準でUTF-8に変更
    '                   （従来のShift-JISもオプションで可能）
    'V77での改良点・・・ファイル入出力画面でキャンセル処理の改善
    'V78での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V79での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V80での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                   （これまでは最長方向の座標軸を採用）
    'V81での改良点・・・構造図画面で条件設定部の選択名称を解析種類で区分け、
    '                   構造図画面で条件設定部の表示辺幅を拡大、
    '                   構造図画面で準定常電磁界の流束入力を廃止
    'V82での改良点・・・サブメニューの中間節点自動計算を廃止
    'V83での改良点・・・解析データのデータチェックに要素共通辺の節点を追加
    'V84での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Hpai As Single = 1.570796 'π/2
    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$                 '計算結果表示データ
    Public Cr$                      'CR+LFのコード
    Public DirOpen$                 '立ち上げ時のディレクトリー名
    Public Fadd$                    '連結するファイル名
    Public Fm$                      '入力データのファイル名
    Public Fy$                      '計算結果のファイル名
    Public Title$                   'タイトル

    Public Timst As Date            '計算開始時刻

    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）
    Public Kaxis As Integer         '作図のＸＹ軸の入換え（0:なし、1:有り）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public KcomSide As Integer      '要素辺の表示有無（0:なし、1:有り）
    Public KcontourPoint As Integer '等高線図で場所･数値表示の有無（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KDrawDatMod As Integer   '構造図でのﾃﾞｰﾀ修正区分（0:個別、1:一括）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public KindBound As Integer     '境界の設定区分（0:自動、1:非自動）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindField As Integer     '解析種類（0:静電界、1:静磁界(電流なし)、
    '                                2:静磁界(電流有り)、3:準定常電磁界）
    Public KindPost As Integer      '後処理の種類（1:等高線図、2:ﾍﾞｸﾄﾙ図、3:端数処理、
    '                                4:主要値表示）
    Public KMenv As Integer         '環境材料番号
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseDset As Integer    'ﾏｳｽ指定要素のﾃﾞｰﾀ項目（1:材料番号、2:境界固定・非固定節点、
    '                                3:流束固定、4:節点負荷）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KnewUFileB As Integer    '最近使用ﾌｧｲﾙBの更新有無
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public KStage As Integer        'ﾃﾞｰﾀ入力の段階番号
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public Mprop As Integer         '材料データの成分数
    Public Nall As Integer          '変数の総数
    Public Nbound, Nboundb As Integer        '境界固定･非固定の節点数
    Public Ndel As Integer          '削除要素数
    Public Ndim As Integer          '１節点当りの変数の数(=1～2)
    Public NDisp As Integer         '等高線の分割数
    Public NelecP, NelecPb As Integer        '電荷・電流負荷の節点数
    Public Nelem As Integer         '要素数
    Public Nflux, Nfluxb As Integer         '流束ｑ固定境界の辺の数
    Public Ngaus As Integer         'ガウス積分点数(3)
    Public NKmax As Integer         '最大繰り返し計算回数
    Public Nmat As Integer          '材料データ数
    Public NMdivMax As Integer      '材料の物性区分数最大値
    Public NoBound As Integer       '境界非固定の節点数（旧ﾃﾞｰﾀ）
    Public Npoint As Integer        '節点数
    Public Ntype As Integer         '問題のタイプ（2:２次元、4:軸対称）
    Public Nvar As Integer          '１要素当りの変数の総数

    Public Carrow As Single         'ベクトルの矢印長さ係数
    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public Cycle As Single          '正弦波周波数
    Public EpsCalc As Single        '収束基準誤差率
    Public VdispMax As Single       '作画変数の最高値
    Public VdispMin As Single       '作画変数の最低値
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public Yupper As Single         '作画領域のY最大値

    Public UFileB(6) As String      '最近使用した解析ﾌｧｲﾙの保存配列

    Public Jrest() As Integer       '境界節点の拘束区分（0:非固定、1:固定）
    Public KAppPoint() As Integer   'ﾃﾞｰﾀ設定の節点
    Public KAppSide(,) As Integer   'ﾃﾞｰﾀ設定の要素指定辺
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KdrawElemOld() As Integer    '変更前の各要素の作画有無（0:なし、1:有り）
    Public KEInside(,) As Integer   '要素の各辺の内部区分（0:外周、1:内部）
    Public KelecP() As Integer      '電荷・電流負荷の節点番号
    Public Koption(3) As Integer    'オプション有無（1:バックアップ、
    '                                2:要素入力画面の節点番号表示、3:節点入力画面の表示）
    Public Kpflux(,) As Integer     '流束ｑ固定境界の３節点番号
    Public KRcomp() As Integer      '計算順序節点の元ﾃﾞｰﾀ節点順序
    Public KSelem(,) As Integer     '各要素面の内部辺指定（0:なし、1：有り）
    Public KsidePoint(3) As Integer '辺の3節点の節点順位
    Public NMdiv() As Integer       '材料の物性区分数
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public Nrest() As Integer       '境界固定･非固定の節点番号
    Public NoRest() As Integer      '境界非固定の節点番号（旧ﾃﾞｰﾀ）
    Public NumMat() As Integer      '各要素の材料種類番号

    Public BMat(,) As Single        '材料の区分磁束密度
    Public DataMat(,) As Single     '材料ﾃﾞｰﾀ
    Public Pnorm() As Single        '流束ｑ
    Public PosGaus(3) As Single     'ｶﾞｳｽ積分点の位置(於要素座標）
    Public QelecP() As Single       '節点負荷の電荷・電流値
    Public Rmyu(,) As Single        '材料の区分透磁率
    Public Tr(8, 9) As Single       'ｶﾞｳｽ点・節点の変換ﾏﾄﾘｯｸｽ
    Public Tu() As Single           'FEM計算の解
    Public Vrest() As Single        '境界節点の固定値
    Public WeightG(3) As Single     'ｶﾞｳｽ積分点の重み
    Public X(), Y() As Single       '節点座標
    Public Xdraw(8), Ydraw(8) As Single     '色塗り多角形の座標
    Public XE(8), YE(8) As Single   '要素計算の配列
    Public Xmouse(), Ymouse() As Single     'ﾏｳｽ位置座標

    Sub AxisCheck(ByRef icheck As Integer, ByRef k1 As Integer, _
        ByRef k2 As Integer, ByRef ratio As Single, ByRef xmin As Single, _
        ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        '         最初に-1が設定された場合は、RZ座標のR最小値は0としない
        'k1,k2･･･要素の開始、終了番号
        'ratio･･･作画直角回転なしの縦横比上限（指定値）
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim i, ie, ik, j As Integer
        Dim b1, b2, ratiob As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = X(i) : xmax = xmin
                        ymin = Y(i) : ymax = ymin
                    Else
                        If X(i) < xmin Then
                            xmin = X(i)
                        ElseIf X(i) > xmax Then
                            xmax = X(i)
                        End If
                        If Y(i) < ymin Then
                            ymin = Y(i)
                        ElseIf Y(i) > ymax Then
                            ymax = Y(i)
                        End If
                    End If
                Next j
            End If
        Next ie
        If Ntype = 4 And icheck >= 0 Then
            xmin = 0
        End If
        b1 = xmax - xmin
        b2 = ymax - ymin
        If b1 = 0 Then
            icheck = 0
        Else
            ratiob = b2 / b1
            If ratiob > ratio Then
                icheck = 1
            Else
                icheck = 0
            End If
        End If

    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf()･･･節点の座標
        Dim i, ie, kind, nb As Integer
        Dim b1, b2, pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        b1 = pa - XaxisMin
        b2 = qa - YaxisMin
        If b2 > b1 Then
            b1 = b2
        End If
        b2 = b1 * 0.00002
        Call RoundPrep(kind, nb, b2)    '丸め準備

        For i = 1 To Npoint
            xbuf(i) = X(i)
            ybuf(i) = Y(i)
        Next i
        For i = 1 To Npoint
            If xbuf(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If ybuf(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'kcbuf()･･･元の節点の変数順序
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        Dim i, ie, ii, j, jj, k, mx As Integer

        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To 7
                i = NNelem(ie, ii)
                For jj = ii + 1 To 8
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie
        mb = Ndim * (mx + 1)

    End Sub

    Sub BoundDupCheck(ByRef nd As Integer)
        '境界固定・非固定の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, nb As Integer
        Dim kbuf(Nbound) As Integer

        nd = 0
        For i = 1 To Nbound - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nbound
                    If Nrest(i) = Nrest(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        If Jrest(ii) <> 0 Then
                            Jrest(i) = Jrest(ii)
                            Vrest(i) = Vrest(ii)
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nbound), jbuf1(Nbound) As Integer
            Dim vbuf(Nbound) As Single
            For i = 1 To Nbound
                nbuf1(i) = Nrest(i)
                jbuf1(i) = Jrest(i)
                vbuf(i) = Vrest(i)
            Next i
            nb = Nbound - nd
            ReDim Nrest(nb), Jrest(nb), Vrest(nb)
            nb = 0
            For i = 1 To Nbound
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    Nrest(nb) = nbuf1(i)
                    Jrest(nb) = jbuf1(i)
                    Vrest(nb) = vbuf(i)
                End If
            Next i
            Nbound = nb
            Nboundb = nb
        End If

    End Sub

    Sub CenterLine(ByVal gr As Graphics, ByVal icheck As Integer)
        '中心線の作画
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'kind･･･ﾋﾟｸﾁｬｰﾎﾞｯｸｽの長さ種類（0:標準、1:大）
        Dim myPen As New Pen(Color.Black)
        myPen.DashStyle = Drawing2D.DashStyle.DashDot
        Select Case icheck
            Case 0
                gr.DrawLine(myPen, Xlower, Ylower, Xlower, Yupper)
            Case 1
                gr.DrawLine(myPen, Xlower, Ylower, Xupper, Ylower)
        End Select

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim a1$, a2$, aa$, aerr1$
        Dim i, ib, ie, im, ip, j, jb, k, kb, kh As Integer
        Dim n1, n2, n3, nb, nx As Integer
        Dim b1, b2 As Single
        Dim abuf$(10)

        a1$ = ""
        a2$ = ""
        aerr1$ = "になっています！"
        num = 0

        If Ntype <> 2 And Ntype <> 4 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  問題のタイプの番号が間違っています！"
        End If
        If KindField < 0 Or KindField > 3 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  解析種類の番号が間違っています！"
        End If
        If Npoint < 8 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  節点数が不足です！"
        End If
        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If Nmat < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  材料データ数が" + Str$(Nmat) + aerr1$
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            nb = NumMat(ie)
            If nb < 1 Or nb > Nmat Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf$(num) = aa$ + " 材料番号が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            For j = 1 To 8
                nb = NNelem(ie, j)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If j < 8 Then
                    For k = j + 1 To 8
                        nx = NNelem(ie, k)
                        If nb = nx Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                            abuf$(num) = aa$ + " ２つの節点番号が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    Next k
                End If
            Next j
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = X(ip) - X(j)
                b2 = Y(ip) - Y(j)
                If b1 = 0 And b2 = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j) + "が"
                    abuf$(num) = aa$ + " 同じ座標位置" + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        '不要節点のﾁｪｯｸ
        Dim kbuf(Npoint) As Integer
        For ie = 1 To Nelem
            For j = 1 To 8
                kbuf(NNelem(ie, j)) = 1
            Next j
        Next ie
        For ip = 1 To Npoint
            If kbuf(ip) = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が使われていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        Next ip

        '要素共通辺のﾁｪｯｸ
        ReDim kbuf(Npoint)
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                ib = NNelem(ie, j)
                If kbuf(ib) = 0 Then
                    kbuf(ib) = 1    '頂点節点に指定
                End If
            Next j
        Next ie
        For ie = 1 To Nelem
            For j = 2 To 8 Step 2
                ib = NNelem(ie, j)
                If kbuf(ib) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ib) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ie

        '材料ﾃﾞｰﾀのﾁｪｯｸ
        For im = 1 To Nmat
            If KindField = 0 Or KindField = 3 Then
                b1 = DataMat(im, 1)
                If b1 <= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                    a1$ = Str$(b1) + aerr1$
                    Select Case KindField
                        Case 0
                            a2$ = " 誘電率が"
                        Case 3
                            a2$ = " 透磁率が"
                    End Select
                    abuf$(num) = aa$ + a2$ + a1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If KindField = 3 Then
                    If DataMat(im, 2) <> 0 And DataMat(im, 3) <> 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                        aa$ = "電流・導電率とも0でありません！"
                        abuf$(num) = aa$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                End If
            Else
                nb = NMdiv(im)
                If nb < 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                    abuf$(num) = aa$ + " 物性区分数が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If KindField = 2 Then
                    If DataMat(im, 1) <> 0 Then
                        For j = 2 To 3
                            If DataMat(im, j) <> 0 Then
                                num = num + 1
                                aa$ = "Error" + LTrim$(Str$(num))
                                aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                                abuf$(num) = aa$ + "電流と磁化が重複しています！"
                                If num = 10 Then
                                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                    Exit Sub
                                End If
                                Exit For
                            End If
                        Next j
                    End If
                End If
                For j = 0 To nb
                    b1 = Rmyu(im, j)
                    If b1 <= 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                        abuf$(num) = aa$ + " 透磁率が" + Str$(b1) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    b1 = BMat(im, j)
                    If j > 0 And b1 <= 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                        abuf$(num) = aa$ + " 磁束密度が" + Str$(b1) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next j
            End If
        Next im

        If Nbound > 0 Then     '節点固定ﾃﾞｰﾀのﾁｪｯｸ
            Select Case KindField
                Case 0
                    a1$ = "  電位固定境界"
                Case 1
                    a1$ = "  磁位固定境界"
                Case 2
                    If Ntype = 2 Then
                        a1$ = "  Ａz固定境界"
                    Else
                        a1$ = "  Ａp固定境界"
                    End If
            End Select
            For ib = 1 To Nbound
                nb = Nrest(ib)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + a1$ + "の番号" + Str$(ib) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nflux > 0 Then   '流束境界のﾁｪｯｸ
            Dim i1, i2, ic, ii, j1, j2, jj, js As Integer
            Dim kmulti(Nelem, 8) As Integer
            For ie = 1 To Nelem
                i1 = NNelem(ie, 8)
                For j = 1 To 8
                    i2 = NNelem(ie, j)
                    For ii = ie + 1 To Nelem
                        j1 = NNelem(ii, 8)
                        For jj = 1 To 8
                            j2 = NNelem(ii, jj)
                            Call NumCheck(ic, i1, i2, j1, j2)   '要素辺の重なりﾁｪｯｸ
                            If ic = 1 Then
                                kmulti(ie, j) = 1
                                kmulti(ii, jj) = 1
                            End If
                            j1 = j2
                        Next jj
                    Next ii
                    i1 = i2
                Next j
            Next ie
            Select Case KindField
                Case 0
                    a1$ = "  電束密度境界"
                Case 1
                    a1$ = "  磁束密度境界"
                Case Else
                    a1$ = "  流束ｑ固定境界"
            End Select
            For ib = 1 To Nflux
                For j = 1 To 3
                    jb = Kpflux(ib, j)
                    If jb < 1 Or jb > Npoint Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + a1$ + "の番号" + Str$(ib) + "の"
                        abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    If j < 3 Then
                        For k = j + 1 To 3
                            kb = Kpflux(ib, k)
                            If jb = kb Then
                                num = num + 1
                                aa$ = "Error" + LTrim$(Str$(num))
                                aa$ = aa$ + a1$ + "の番号" + Str$(ib) + "で"
                                abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "が重複しています！"
                                If num = 10 Then
                                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                    Exit Sub
                                End If
                            End If
                        Next k
                    End If
                Next j
                n1 = Kpflux(ib, 1)
                n2 = Kpflux(ib, 2)
                n3 = Kpflux(ib, 3)
                Call SideSearch(kh, js, n1, n2, n3) '辺の探索
                If kh = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + a1$ + "の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 辺の指定が間違っています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                ElseIf kmulti(kh, js) = 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + a1$ + "の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 内部辺が指定されています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If NelecP > 0 Then
            If KindField = 0 Then
                a1$ = "  節点電荷"
            Else
                a1$ = "  節点電流"
            End If
            For i = 1 To NelecP
                nb = KelecP(i)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + a1$ + "の番号" + Str$(i) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next i
        End If

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "EleMag2D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ntype = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindField = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindBound = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nbound = dat(1)

        If KindField = 3 Then
            Nflux = 0
        Else
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Nflux = dat(1)
        End If
        If KindField = 1 Or KindField = 3 Then
            NelecP = 0
        Else
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NelecP = dat(1)
        End If
        If KindField > 2 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Cycle = dat(1)
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ngaus = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Mprop = dat(1)

        ReDim NNelem(Nelem, 8), NumMat(Nelem)
        ReDim X(Npoint), Y(Npoint)
        ReDim NMdiv(Nmat), DataMat(Nmat, Mprop)
        ReDim Nrest(Nbound), Jrest(Nbound), Vrest(Nbound)
        ReDim NoRest(NoBound)
        ReDim Kpflux(Nflux, 3), Pnorm(Nflux)
        ReDim KelecP(NelecP), QelecP(NelecP)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 8
                NNelem(i, j) = dat(j + 1)
            Next j
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            X(i) = dat(1)
            Y(i) = dat(2)
        Next i

        aa$ = sr.ReadLine
        If KindField = 1 Or KindField = 2 Then
            NMdivMax = 0
            For i = 1 To Nmat
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NMdiv(i) = dat(1)
                If NMdiv(i) > NMdivMax Then
                    NMdivMax = NMdiv(i)
                End If
            Next i
            ReDim BMat(Nmat, NMdivMax), Rmyu(Nmat, NMdivMax)
            aa$ = sr.ReadLine
            For i = 1 To Nmat
                aa$ = sr.ReadLine
                If KindField = 2 Then
                    aa$ = sr.ReadLine
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    For j = 1 To Mprop
                        DataMat(i, j) = dat(j - 1)
                    Next j
                End If
                aa$ = sr.ReadLine
                For j = 0 To NMdiv(i)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    BMat(i, j) = dat(1)
                    Rmyu(i, j) = dat(2)
                Next j
            Next i
        Else
            For i = 1 To Nmat
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 1 To Mprop
                    DataMat(i, j) = dat(j)
                Next
            Next i
        End If

        If Nbound > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To Nbound
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Nrest(i) = dat(1)
                Jrest(i) = dat(2)
                Vrest(i) = dat(3)
            Next i
        End If

        If Nflux > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nflux
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Kpflux(i, 1) = dat(1)
                Kpflux(i, 2) = dat(2)
                Kpflux(i, 3) = dat(3)
                Pnorm(i) = dat(4)
            Next i
        End If

        If NelecP > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To NelecP
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KelecP(i) = dat(1)
                QelecP(i) = dat(2)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, a2$, aa$
        Dim i, j As Integer

        a2$ = ""
        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "EleMag2D"
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "問題のタイプ(2:２次元、4:軸対称体)" + "," + Str(Ntype)
        sw.WriteLine(aa$)
        Select Case KindField
            Case 0
                aa$ = "解析種類(0:静電界)"
            Case 1
                aa$ = "解析種類(1:静磁界(電流なし))"
            Case 2
                aa$ = "解析種類(2:静磁界(電流有り))"
            Case 3
                aa$ = "解析種類(3:準定常電磁界)"
        End Select
        aa$ = aa$ + "," + Str(KindField)
        sw.WriteLine(aa$)
        aa$ = "境界の設定区分（0:自動、1:非自動）" + "," + Str(KindBound)
        sw.WriteLine(aa$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        Select Case KindField
            Case 0
                aa$ = "電位固定･非固定の節点数"
                a1$ = "電束固定境界の辺の数"
                a2$ = "電荷の節点数"
            Case 1
                aa$ = "磁位固定･非固定の節点数"
                a1$ = "磁束固定境界の辺の数"
            Case Else
                a1$ = "流束ｑ固定境界の辺の数"
                If Ntype = 2 Then
                    aa$ = "Ａz固定･非固定の節点数"
                Else
                    aa$ = "Ａp固定･非固定の節点数"
                End If
                a2$ = "電流の節点数"
        End Select
        Select Case KindField
            Case 0
                Ndim = 1 : Mprop = 2
            Case 1
                Ndim = 1 : Mprop = 1
                NelecP = 0
            Case 2
                'Ndim = 1 : Mprop = 2
                Ndim = 1 : Mprop = 3
            Case 3
                Nflux = 0
                NelecP = 0
                Ndim = 2 : Mprop = 3
        End Select

        aa$ = aa$ + "," + Str(Nbound)
        sw.WriteLine(aa$)
        If KindField < 3 Then
            aa$ = a1$ + "," + Str(Nflux)
            sw.WriteLine(aa$)
        End If
        If KindField = 0 Or KindField = 2 Then
            aa$ = a2$ + "," + Str(NelecP)
            sw.WriteLine(aa$)
        End If
        If KindField > 2 Then
            aa$ = "正弦波周波数(Hz)" + "," + Str(Cycle)
            sw.WriteLine(aa$)
        End If
        aa$ = "ガウス積分点数" + "," + Str(Ngaus)
        sw.WriteLine(aa$)
        aa$ = "１節点当り変数の数" + "," + Str(Ndim)
        sw.WriteLine(aa$)
        aa$ = "材料データの成分数" + "," + Str(Mprop)
        sw.WriteLine(aa$)

        sw.WriteLine("要素データ (要素番号、材料番号、8節点番号)")
        For i = 1 To Nelem
            aa$ = Str$(i) + ",   " + Str$(NumMat(i))
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        If Ntype = 2 Then
            aa$ = "節点データ（節点番号、Ｘ座標、Ｙ座標）"
        Else
            aa$ = "節点データ（節点番号、Ｒ座標、Ｚ座標）"
        End If
        sw.WriteLine(aa$)
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            sw.WriteLine(aa$)
        Next i

        If KindField = 1 Or KindField = 2 Then
            sw.WriteLine("各材料の透磁率区分数（材料番号、区分数)")
            For i = 1 To Nmat
                aa$ = "  " + Str$(i) + ",  " + Str$(NMdiv(i))
                sw.WriteLine(aa$)
            Next i
            sw.WriteLine("各材料の物性データ")
            For i = 1 To Nmat
                aa$ = "材料番号" + "," + Str(i)
                sw.WriteLine(aa$)
                If KindField = 2 Then
                    aa$ = "  垂直電流密度"
                    If Ntype = 2 Then
                        aa$ = aa$ + "、磁化Ｍx、磁化Ｍy"
                    Else
                        aa$ = aa$ + "、磁化Ｍr、磁化Ｍz"
                    End If
                    sw.WriteLine(aa$)
                    aa$ = "  " + Str(DataMat(i, 1))
                    For j = 2 To 3
                        aa$ = aa$ + ",  " + Str(DataMat(i, j))
                    Next j
                    sw.WriteLine(aa$)
                End If
                sw.WriteLine("  区分番号、磁束密度、透磁率")
                For j = 0 To NMdiv(i)
                    aa$ = "    " + Str$(j) + ",  " + Str$(BMat(i, j))
                    aa$ = aa$ + ",  " + Str$(Rmyu(i, j))
                    sw.WriteLine(aa$)
                Next j
            Next i
        Else
            Select Case KindField
                Case 0
                    sw.WriteLine("材料データ（材料番号、誘電率、電荷密度）")
                Case 3
                    aa$ = "材料データ（材料番号、透磁率、導電率、垂直電流密度）"
                    sw.WriteLine(aa$)
            End Select
            For i = 1 To Nmat
                aa$ = Str$(i)
                For j = 1 To Mprop
                    aa$ = aa$ + ",   " + Str$(DataMat(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nbound > 0 Then
            Select Case KindField
                Case 0
                    aa$ = "＜電位固定･非固定の境界＞"
                    a1$ = "固定番号､節点番号､電位拘束､電位値"
                Case 1
                    aa$ = "＜磁位固定･非固定の境界＞"
                    a1$ = "固定番号､節点番号､磁位拘束､拘束値"
                Case Else
                    If Ntype = 2 Then
                        aa$ = "＜Ａz固定･非固定の境界＞"
                        a1$ = "固定番号､節点番号､Ａz拘束､Ａz値"
                    Else
                        aa$ = "＜Ａp固定･非固定の境界＞"
                        a1$ = "固定番号､節点番号､Ａp拘束､Ａp値"
                    End If
            End Select
            sw.WriteLine(aa$)
            sw.WriteLine(a1$)
            For i = 1 To Nbound
                aa$ = Str$(i) + ",   " + Str$(Nrest(i))
                aa$ = aa$ + ",   " + Str$(Jrest(i))
                aa$ = aa$ + ",   " + Str$(Vrest(i))
                sw.WriteLine(aa$)
            Next i
        End If

        If Nflux > 0 Then
            Select Case KindField
                Case 0
                    aa$ = "電束密度境界（境界番号、辺の３節点番号、"
                    aa$ = aa$ + "電束密度ｑ（単位面積当り））"
                Case 1
                    aa$ = "磁束密度境界（境界番号、辺の３節点番号、"
                    aa$ = aa$ + "磁束密度ｑ（単位面積当り））"
                Case Else
                    aa$ = "流束ｑ固定境界（境界番号、辺の３節点番号、"
                    aa$ = aa$ + "流束ｑ（単位面積当り））"
            End Select
            sw.WriteLine(aa$)
            For i = 1 To Nflux
                aa$ = Str$(i)
                For j = 1 To 3
                    aa$ = aa$ + ",   " + Str$(Kpflux(i, j))
                Next j
                aa$ = aa$ + ",   " + Str$(Pnorm(i))
                sw.WriteLine(aa$)
            Next i
        End If

        If NelecP > 0 Then
            If KindField = 0 Then
                aa$ = "節点電荷データ（電荷番号、節点番号、"
                aa$ = aa$ + "電荷量（単位厚さ当り））"
            Else
                If Ntype = 2 Then
                    aa$ = "節点電流データ（電流番号、節点番号、"
                    aa$ = aa$ + "電流値（Ｚ軸方向））"
                Else
                    aa$ = "節点電流データ（電流番号、節点番号、"
                    aa$ = aa$ + "電流値（Ｚ軸左回り円周方向））"
                End If
            End If
            sw.WriteLine(aa$)

            For i = 1 To NelecP
                aa$ = Str$(i)
                aa$ = aa$ + ",   " + Str$(KelecP(i))
                aa$ = aa$ + ",   " + Str$(QelecP(i))
                sw.WriteLine(aa$)
            Next i
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawNodeVar(ByRef gr As Graphics, ByVal aa As String, _
    ByVal xb As Single, ByVal yb As Single)
        '節点変数の描画
        'aa･･･変数のテキスト
        'xb,yb･･･描画位置
        Dim xbw, ybw As Single
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", 10, FontStyle.Bold)

        xbw = xb - 1
        ybw = yb
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb
        ybw = yb - 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        ybw = yb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        gr.DrawString(aa$, fnt2, Brushes.Purple, xb, yb)

    End Sub

    Sub DrawScale(ByRef icheck As Integer, ByVal dpixb As Single, _
        ByVal dpiyb As Single, ByVal xlen As Single, ByVal ylen As Single, _
        ByVal xmin As Single, ByVal xmax As Single, _
        ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        Select Case icheck
            Case 0
                If Cfix < 1 Then
                    If b1 <> 0 Then
                        cx = xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2 / Cfix
                    End If
                Else
                    If b1 <> 0 Then
                        cx = Cfix * xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2
                    End If
                End If
                If b1 = 0 Then
                    XYscale = cy
                ElseIf b2 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
            Case 1
                If Cfix < 1 Then
                    If b2 <> 0 Then
                        cx = xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1 / Cfix
                    End If
                Else
                    If b2 <> 0 Then
                        cx = Cfix * xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1
                    End If
                End If
                If b2 = 0 Then
                    XYscale = cy
                ElseIf b1 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
        End Select

    End Sub

    Sub Elem2DPrep()
        '構造図作画準備
        Dim ie As Integer
        Dim pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        KE1 = 1
        KE2 = Nelem
        KnumDisp = 0
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        Call OutSideCheck() '外周辺のﾁｪｯｸ

    End Sub

    Sub ElemSearch2(ByRef k1 As Integer, ByRef k2 As Integer)
        '拡大作画する要素の探索
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        Dim ib, ic, ie, j, kmax, kmin As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    b1 = X(ib) - Xmouse(1)
                    b2 = X(ib) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        If Kaxis = 0 Then
                            b1 = Y(ib) - Ymouse(2)
                            b2 = Y(ib) - Ymouse(1)
                        Else
                            b1 = Y(ib) - Ymouse(1)
                            b2 = Y(ib) - Ymouse(2)
                        End If
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 8 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If
    End Sub

    Sub ElemSearch3(ByVal k1 As Integer, ByVal k2 As Integer, ByRef nb As Integer)
        '矩形領域の辺の探索
        'k1,k2･･･要素の開始、終了番号
        'nb･･･適合辺の数
        Dim ib, ie, j, jj, k As Integer
        Dim b1, b2 As Single
        Dim kbuf(3) As Integer

        nb = 0
        For ie = 1 To Nelem
            For j = 1 To 4
                KAppSide(ie, j) = 0
            Next j
        Next ie
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    If KEInside(ie, j) = 0 Then
                        jj = 2 * j - 1
                        kbuf(1) = NNelem(ie, jj)
                        kbuf(2) = NNelem(ie, jj + 1)
                        jj = jj + 2
                        If jj > 8 Then
                            jj = 1
                        End If
                        kbuf(3) = NNelem(ie, jj)
                        For k = 1 To 3
                            ib = kbuf(k)
                            b1 = X(ib) - Xmouse(1)
                            b2 = X(ib) - Xmouse(2)
                            If b1 >= 0 And b2 <= 0 Then
                                If Kaxis = 0 Then
                                    b1 = Y(ib) - Ymouse(2)
                                    b2 = Y(ib) - Ymouse(1)
                                Else
                                    b1 = Y(ib) - Ymouse(1)
                                    b2 = Y(ib) - Ymouse(2)
                                End If
                                If b1 >= 0 And b2 <= 0 Then
                                    If k = 3 Then
                                        KAppSide(ie, j) = 1
                                        nb = nb + 1
                                    End If
                                Else
                                    Exit For
                                End If
                            Else
                                Exit For
                            End If
                        Next k
                    End If
                Next j
            End If
        Next ie

    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByRef abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub FDatDupCheck(ByRef nd As Integer)
        '電流・電荷節点の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, nb As Integer
        Dim kbuf(NelecP) As Integer

        nd = 0
        For i = 1 To NelecP - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To NelecP
                    If KelecP(i) = KelecP(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        QelecP(i) = QelecP(ii)
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(NelecP) As Integer
            Dim buf2(NelecP) As Single
            For i = 1 To NelecP
                nbuf1(i) = KelecP(i)
                buf2(i) = QelecP(i)
            Next i
            nb = NelecP - nd
            ReDim KelecP(nb), QelecP(nb)
            nb = 0
            For i = 1 To NelecP
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    KelecP(nb) = nbuf1(i)
                    QelecP(nb) = buf2(i)
                End If
            Next i
            NelecP = nb
            NelecPb = nb
        End If

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub FluxDupCheck(ByRef nd As Integer)
        '流束の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nflux) As Integer

        nd = 0
        For i = 1 To Nflux - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nflux
                    If Kpflux(i, 1) = Kpflux(ii, 1) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        Pnorm(i) = Pnorm(ii)
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nflux, 3) As Integer
            Dim buf2(Nflux) As Single
            For i = 1 To Nflux
                For j = 1 To 3
                    nbuf1(i, j) = Kpflux(i, j)
                Next j
                buf2(i) = Pnorm(i)
            Next i
            nb = Nflux - nd
            ReDim Kpflux(nb, 3), Pnorm(nb)
            nb = 0
            For i = 1 To Nflux
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 3
                        Kpflux(nb, j) = nbuf1(i, j)
                    Next j
                    Pnorm(nb) = buf2(i)
                End If
            Next i
            Nfluxb = nb
            Nflux = nb
        End If

    End Sub

    Sub GausSet()
        'ｶﾞｳｽ点の設定
        PosGaus(3) = 0.77459667 : PosGaus(1) = -PosGaus(3)
        PosGaus(2) = 0
        WeightG(1) = 0.55555556 : WeightG(3) = WeightG(1)
        WeightG(2) = 0.88888889

    End Sub

    Sub InnerCheck(ByVal k1 As Integer, ByVal k2 As Integer)
        '内部辺ﾁｪｯｸ
        'k1,k2･･･開始・終了要素番号
        Dim ib, ic, ie, i1, i2, ik, j1, j2, k, kk As Integer
        Dim kbuf(3) As Integer
        Dim ksbuf(Nelem, 4) As Integer    'ﾁｪｯｸ済みﾌﾗｯｸﾞ
        For ie = k1 To k2 - 1
            If KdrawElem(ie) > 0 Then
                For k = 1 To 4
                    If ksbuf(ie, k) = 0 Then
                        Call SideSet(ie, k, KsidePoint)   '辺の設定
                        ik = 0
                        i1 = KsidePoint(1)
                        i2 = KsidePoint(3)
                        For ib = ie + 1 To k2
                            If KdrawElem(ib) > 0 Then
                                For kk = 1 To 4
                                    If ksbuf(ib, kk) = 0 Then
                                        Call SideSet(ib, kk, kbuf)
                                        '辺の設定
                                        j1 = kbuf(1)
                                        j2 = kbuf(3)
                                        Call NumCheck(ic, i1, i2, j1, j2)
                                        '要素辺の重なりﾁｪｯｸ

                                        If ic = 1 Then
                                            ik = 1
                                            KSelem(ie, k) = 1
                                            KSelem(ib, kk) = 1
                                            ksbuf(ie, k) = 1
                                            ksbuf(ib, kk) = 1
                                            Exit For
                                        End If
                                    End If
                                Next kk
                                If ik = 1 Then
                                    Exit For
                                End If
                            End If
                        Next ib
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub MouseCalc(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            If Kaxis = 0 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                yb = (xa - Xlower) / XYscale + YaxisMin
            End If
        Else
            If Kaxis = 0 Then
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale + XaxisMin
                yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
            End If
        End If

    End Sub

    Sub MouseCalc2()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                If Kaxis = 0 Then
                    xb = (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                    yb = (xa - Xlower) / XYscale + YaxisMin
                End If
            Else
                If Kaxis = 0 Then
                    xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale + XaxisMin
                    yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
                End If
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MousePointSearch(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single)
        'ﾏｳｽ点の節点の探索
        'k1,k2･･･要素の開始、終了番号
        'ky･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点のﾓﾃﾞﾙ座標位置
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        b1 = X(ib) - xb
                        b2 = Y(ib) - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub NumCheck(ByRef ic As Integer, ByRef i1 As Integer, _
        ByRef i2 As Integer, ByRef j1 As Integer, _
        ByRef j2 As Integer)
        '要素辺の重なりのﾁｪｯｸ
        'ic･･･要素辺の重なり有無（0:なし、1:有り）
        'i1･･･第1の辺の一端の節点番号
        'i2･･･第1の辺の他端の節点番号
        'j1･･･第2の辺の一端の節点番号
        'j2･･･第2の辺の他端の節点番号
        ic = 0
        If i1 = j1 Then
            If i2 = j2 Then
                ic = 1
            End If
        ElseIf i1 = j2 Then
            If i2 = j1 Then
                ic = 1
            End If
        End If

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\ElMa2DMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            For i = 1 To 3
                Koption(i) = Val(sr.ReadLine)
            Next i
            Kscreen = Val(sr.ReadLine)
            KcomSide = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\ElMa2DMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 3
            sw.WriteLine(Koption(i))
        Next i
        sw.WriteLine(Kscreen)
        sw.WriteLine(KcomSide)
        sw.Close()

    End Sub

    Sub OutlineDraw(ByRef gr As Graphics, ByVal k1 As Integer, ByVal k2 As Integer)
        '指定要素範囲の外形の作画
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'k1･･･作画初めの要素番号
        'k2･･･作画終りの要素番号
        Dim i, ie, j, j1, j2, k As Integer
        Dim xi, xj, yi, yj As Single

        Call OutSideCheck()  '外周辺のﾁｪｯｸ

        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    If KEInside(ie, j) = 0 Then
                        For k = 1 To 2
                            If k = 1 Then
                                j1 = 2 * j - 1
                                j2 = j1 + 1
                            Else
                                j1 = j2
                                j2 = j1 + 1
                                If j2 > 8 Then
                                    j2 = 1
                                End If
                            End If
                            i = NNelem(ie, j1)
                            xi = X(i)
                            yi = Y(i)
                            Call XYCalc2(Kaxis, xi, yi)  'X,Yの作画座標計算
                            i = NNelem(ie, j2)
                            xj = X(i)
                            yj = Y(i)
                            Call XYCalc2(Kaxis, xj, yj)  'X,Yの作画座標計算
                            gr.DrawLine(Pens.Black, xi, yi, xj, yj)
                        Next k
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub OutSideCheck()
        '外周辺のﾁｪｯｸ
        Dim i2, ie, ii, j, j2, jj As Integer
        ReDim KEInside(Nelem, 4)

        For ie = 1 To Nelem - 1
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    i2 = NNelem(ie, 2 * j)
                    For ii = ie + 1 To Nelem
                        If KdrawElem(ii) > 0 Then
                            For jj = 1 To 4
                                j2 = NNelem(ii, 2 * jj)
                                If i2 = j2 Then
                                    KEInside(ie, j) = 1
                                    KEInside(ii, jj) = 1
                                End If
                            Next jj
                        End If
                    Next ii
                Next j
            End If
        Next ie

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i) + 0.001
            xydraw(i - 1).Y = Ydraw(i) + 0.001
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PointSearch(ByRef nb As Integer)
        '矩形領域の節点の探索
        'nb･･･適合節点の数
        Dim i As Integer
        Dim b1, b2 As Single
        Dim kbuf(3) As Integer

        nb = 0
        For i = 1 To Npoint
            KAppPoint(i) = 0
            b1 = X(i) - Xmouse(1)
            b2 = X(i) - Xmouse(2)
            If b1 >= 0 And b2 <= 0 Then
                If Kaxis = 0 Then
                    b1 = Y(i) - Ymouse(2)
                    b2 = Y(i) - Ymouse(1)
                Else
                    b1 = Y(i) - Ymouse(1)
                    b2 = Y(i) - Ymouse(2)
                End If
                If b1 >= 0 And b2 <= 0 Then
                    KAppPoint(i) = 1
                    nb = nb + 1
                End If
            End If
        Next i

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub SideSet(ByVal ie As Integer, ByVal k As Integer, _
        ByRef kbuf() As Integer)
        '辺の設定
        'ib･･･要素番号
        'k･･･辺番号
        'kbuf()･･･辺節点の順位番号
        Dim j, kk, kx As Integer
        kk = 2 * k - 1
        kbuf(1) = kk
        kbuf(2) = kk + 1
        kx = kk + 2
        If kx > 8 Then
            kx = 1
        End If
        kbuf(3) = kx

        For j = 1 To 3
            kbuf(j) = NNelem(ie, kbuf(j))
        Next j

    End Sub

    Private Sub SideSearch(ByRef kh As Integer, ByRef js As Integer,
        ByVal n1 As Integer, ByVal n2 As Integer, ByVal n3 As Integer)
        '辺の探索
        'kh･･･要素番号（0はなし、1:有り）
        'js･･･一致辺の中間節点順序番号
        'n1,n2,n3･･･3節点の番号
        Dim ie, j, jx, k1, k2, k3 As Integer
        kh = 0
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                k1 = NNelem(ie, j)
                If k1 = n1 Then
                    k2 = NNelem(ie, j + 1)
                    If j < 7 Then
                        jx = j + 2
                    Else
                        jx = 1
                    End If
                    k3 = NNelem(ie, jx)
                    If k2 = n2 And k3 = n3 Then
                        kh = ie
                        js = j + 1
                        Exit For
                    End If
                End If
            Next j
            If kh > 0 Then
                Exit For
            End If
        Next ie

    End Sub

    Sub TimCal(ByVal ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\EleMag2D.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\EleMag2D.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub TrSet()
        'ｶﾞｳｽ点・節点への変換ﾏﾄﾘｯｸｽ設定
        Dim i As Integer
        Dim b1, b2, b3, b4, b5, b6 As Single

        b1 = 1.93694 : b2 = -0.4858871 : b3 = 0.02777768 : b4 = -0.5555556
        b5 = 0.3747762 : b6 = -0.2147176
        For i = 1 To 7 Step 2
            Tr(i, 1) = b3 : Tr(i, 3) = b3 : Tr(i, 7) = b3 : Tr(i, 9) = b3
            Tr(i, 5) = b4
        Next i
        Tr(1, 1) = b1 : Tr(1, 2) = b2 : Tr(1, 4) = b2
        Tr(1, 6) = b5 : Tr(1, 8) = b5 : Tr(1, 9) = b6
        Tr(3, 2) = b5 : Tr(3, 3) = b6 : Tr(3, 4) = b2
        Tr(3, 6) = b5 : Tr(3, 7) = b1 : Tr(3, 8) = b2
        Tr(5, 1) = b6 : Tr(5, 2) = b5 : Tr(5, 4) = b5
        Tr(5, 6) = b2 : Tr(5, 8) = b2 : Tr(5, 9) = b1
        Tr(7, 2) = b2 : Tr(7, 3) = b1 : Tr(7, 4) = b5
        Tr(7, 6) = b2 : Tr(7, 7) = b6 : Tr(7, 8) = b5

        b1 = 0.1666667 : b2 = -0.3333336 : b3 = 0.1666668 : b4 = 1.145497
        b5 = 0 : b6 = -0.1454973
        For i = 2 To 8 Step 2
            Tr(i, 1) = b1 : Tr(i, 3) = b1 : Tr(i, 7) = b1 : Tr(i, 9) = b1
            Tr(i, 5) = b5
        Next i
        Tr(2, 2) = b2 : Tr(2, 3) = b3 : Tr(2, 4) = b4
        Tr(2, 6) = b6 : Tr(2, 8) = b2 : Tr(2, 9) = b3
        Tr(4, 1) = b3 : Tr(4, 2) = b6 : Tr(4, 3) = b3
        Tr(4, 4) = b2 : Tr(4, 6) = b2 : Tr(4, 8) = b4
        Tr(6, 1) = b3 : Tr(6, 2) = b2 : Tr(6, 4) = b6
        Tr(6, 6) = b4 : Tr(6, 7) = b3 : Tr(6, 8) = b2
        Tr(8, 2) = b4 : Tr(8, 4) = b2 : Tr(8, 6) = b2
        Tr(8, 7) = b3 : Tr(8, 8) = b6 : Tr(8, 9) = b3

    End Sub

    Sub UFileAddB()
        'UFileBの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileB(i) = UFileB(0) Then
                For j = i - 1 To 0 Step -1
                    UFileB(j + 1) = UFileB(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileB(i + 1) = UFileB(i)
            Next i
        End If
        UseFileBOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileBIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\ElMa2DUFileB.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileB(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileBOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\ElMa2DUFileB.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileB(i) <> "" Then
                sw.WriteLine(UFileB(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub XYCalc2(ByVal icheck As Integer, ByRef xb As Single, _
        ByRef yb As Single)
        'X,Yの作画座標計算
        'icheck･･･作画時のXY入れ替えの有無（0:なし、1:有り）
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            If icheck = 0 Then
                xb = b1 * XYscale + Xlower
                yb = Yupper - b2 * XYscale * Cfix
            Else
                xb = b2 * XYscale + Xlower
                yb = b1 * XYscale * Cfix + Ylower
            End If
        Else
            If icheck = 0 Then
                xb = b1 * XYscale / Cfix + Xlower
                yb = Yupper - b2 * XYscale
            Else
                xb = b2 * XYscale / Cfix + Xlower
                yb = b1 * XYscale + Ylower
            End If
        End If
    End Sub

End Module
